#!/usr/bin/perl -w

package Websteuer::Base;

use strict;
use Exporter;
use vars qw ( @ISA @EXPORT );

# Crypto
use Crypt::TripleDES;
use Crypt::RSA;
use Digest::MD5 qw(md5_base64);
use MIME::Base64;

# Templates
use Template;

# eigene Module
use Neko::config;
use db_sqlite;

# debugging on/off 1/0
my $d = 0;

# ----------------------------------------------------------------------
# these Lines are inserted by make - touch carefully!
# START automatic configuration ----------------------------------------

# my $configfile = '/home/neko/.html-data/projects/www.nekosoft.de/websteuer_neu/websteuer/conf/default.conf';
my $configfile = '/Users/neko/projects/websteuer_neu/websteuer/conf/default_mac.conf';

#   END automatic configuration ----------------------------------------
# ----------------------------------------------------------------------


my %conf = read_config($configfile);

if ( $conf{'errorcode'} ) { print STDERR $conf{'errorcode'}.' - '.  $conf{'errormsg'}."\n"; }

my $TEMPLATES_DIR = $conf{'data'}{'TEMPLATES_DIR'} || '';
my $ARCHIV_DIR = $conf{'data'}{'ARCHIV_DIR'} || '';
my $USER_CONFIG_DIR = $conf{'data'}{'USER_CONFIG_DIR'} || '';
my $DB_FILE = $conf{'data'}{'DB_FILE'} || '';
my $TMP_DIR = $conf{'data'}{'TMP_DIR'} || '';
my $LOG_DIR = $conf{'data'}{'LOG_DIR'} || '';
my $COALA_CONFIG_FILE = $conf{'data'}{'COALA_CONFIG_FILE'} || '';
my $JAVA = $conf{'data'}{'JAVA'} || '';
my $SEND_ANTRAG = $conf{'data'}{'SEND_ANTRAG'} || ''; # external JavaModul 
                                              # crypt xml, send, decrypt result

@ISA = qw(Exporter);
@EXPORT = qw( 
              check_antrag
	      decrypt
	      encrypt
	      gen_antrag
	      get_user_permission
	      init_antrag
              make_md5_hash
	      normalize_data
	      parse_herstellerID
              read_antrag
              read_user_config
	      send_antrag
	      sign_data
	      verify_signature
	    );

# ----------------------------------------------------------------------
# Template bauen
# INCLUDE_PATH => $TEMPLATES_DIR,  # or list ref
my $template_config = {
        POST_CHOMP   => 0,               # cleanup whitespace
         PRE_CHOMP   => 0,               # cleanup whitespace
              TRIM   => 0,               # cleanup whitespace
        INCLUDE_PATH => $TEMPLATES_DIR,  # or list ref
           START_TAG => "<% ",           # anpassung an space if_config
             END_TAG => " %>"
};
# ----------------------------------------------------------------------


# ----------------------------------------------------------------------
# HerstellerID entschluesseln:
# 1. md5 hashkey base64 der entsprechenden Files
# 2. als Schluessel fuer TrippleDES Entschluesselung verwenden
# 3. Ergebnis-Datei hat Format: 
#     HerstellerID: <ID>
#     Version: <Versionsnummer>
#     Hersteller: NekoSoft Simone Demmel
#     Key: <verwendeter Schluessel>
#     Product: <Produktname>
#     ENDE
#     <signatur>
# 4. bei Bedarf/Verdacht die Signatur gegen den pub.Key pruefen    
# 
# HerstellerID verschluesseln:
# 1. md5 hashkey base64 der entsprechenden Files
# 2. Datei erstellen im Format:
#     HerstellerID: <ID>
#     Version: <Versionsnummer>
#     Hersteller: NekoSoft Simone Demmel
#     Key: <verwendeter Schluessel>
#     Product: <Produktname>
#     ENDE
# 3. RSA Signatur ueber die Datei mit priv.Key erstellen und hinten
#    anhaengen
# 4. MD5Hashkey als Schluessel fuer TrippleDES Verschluesselung verwenden
# 
# my %md5_hash = make_md5_hash(@dateien);
#       $md5_hash{'key'}
#       $md5_hash{'errorcode'}
#       $md5_hash{'errormsg'}
# my %decrypt = decrypt($iddatei, $key);
#       $decrypt{'daten'}
#       $decrypt{'errorcode'}
#       $decrypt{'errormsg'}
# my %encrypt = encrypt($iddatei, $key);
#       $encrypt{'daten'}
#       $encrypt{'errorcode'}
#       $encrypt{'errormsg'}
# my %sign_data = sign_data($data, $privKey)
#       $sign_data{'signed_data'}
#       $sign_data{'errorcode'}
#       $sign_data{'errormsg'}
# my %verify_sig = verify_signature($data, $pubKey, $signatur)
#       $verify_sig{'verified'}
#       $verify_sig{'errorcode'}
#       $verify_sig{'errormsg'}
# my %datenfeld = parse_herstellerID($data);
#       $datenfeld{'HerstellerID'}
#       $datenfeld{'Version'}
#       $datenfeld{'Hersteller'}
#       $datenfeld{'Key'}
#       $datenfeld{'errorcode'}
#       $datenfeld{'errormsg'}
#          
# externe Perlmodule:
# Crypt::TripleDES
# Crypt::RSA
# Digest::MD5
# MIME::Base64(?)
#
# mit %INC (Modulname als Schluessel) zu pruefende Dateien holen
 
# my %md5_hash = make_md5_hash(@dateien);
# use Digest
sub make_md5_hash
{
    my @dateien = @_;
    my $hashme = '';
    my %md5_hash = ();
    $md5_hash{'key'} = '';
    $md5_hash{'errorcode'} = 0;
    $md5_hash{'errormsg'} = '';

    # read all files (dateien) in one string
    # get pathes from %INC
    foreach my $datei (@dateien)
    {
	# no file
	if ( !$datei ) { next; }
	my $oeffne = '';

        if ( $INC{$datei} )
	{ $oeffne =  $INC{$datei}; }
	else { $oeffne = $datei; }

	open ( IN, "$oeffne" ) || do {
	         $md5_hash{'errorcode'} = 1;
		 $md5_hash{'errormsg'} .= "can not open $oeffne : $!\n";
		 next;
	     };
	while ( <IN> ) { $hashme .= $_; }
	close (IN);
    }
    
    # and make md5 haskey
    $md5_hash{'key'} = md5_base64($hashme);

    return %md5_hash;
}

# my %decrypt = decrypt($iddatei, $key);
# use Crypt::TripleDES;
sub decrypt
{
    my $file = shift || '';
    my $key = shift || '';
    my $decryptme = '';
    my %decrypt = ();

    $decrypt{'daten'} = '';
    $decrypt{'errorcode'} = 0;
    $decrypt{'errormsg'} = '';

    open ( IN , "$file" ) || do 
         {
             $decrypt{'errorcode'} = 1;
             $decrypt{'errormsg'} = "cannot open file $file : $!\n";
             return %decrypt;
	 };

    while (<IN>) { $decryptme .= $_; }
    close (IN);

    my $des = new Crypt::TripleDES;
    my $daten = decode_base64($decryptme);
    $decrypt{'daten'} = $des->decrypt3($daten, $key);

    return %decrypt;
}

# for the owner to secure his id-file
# my %encrypt = encrypt($iddatei, $key);
# use Crypt::TripleDES;
sub encrypt
{
    my $file = shift || '';
    my $key = shift || '';
    my $cryptme = '';
    my %encrypt = ();
    
    $encrypt{'daten'} = '';
    $encrypt{'errorcode'} = 0;
    $encrypt{'errormsg'} = '';

    open ( IN , "$file" ) || do 
         {
             $encrypt{'errorcode'} = 1;
             $encrypt{'errormsg'} = "cannot open file $file : $!\n";
             return %encrypt;
	 };

    while (<IN>) { $cryptme .= $_; }
    close (IN);

    my $des = new Crypt::TripleDES;
    my $daten = $des->encrypt3($cryptme, $key);
    $encrypt{'daten'} = encode_base64($daten);

    return %encrypt;
}

# my %sign_data = sign_data($data, $privKey)
sub sign_data
{
    # das ist ein einziger Scheiss mit dem Crypto-Muell auf CPAN.
    # wir 'signieren' erst mal mit einem hash ueber sich selbst.

    my $data = shift || '';
    my $privKey = shift || '';
    my %sign_data = ();
    $sign_data{'signature'} = '';
    $sign_data{'errorcode'} = 0;
    $sign_data{'errormsg'} = '';

    $sign_data{'signature'} = md5_base64($data);

    return %sign_data;
}

# my %verify_sig = verify_signature($data, $pubKey, $signatur)
sub verify_signature
{
    my $data = shift || '';
    my $pubKey = shift || '';
    my $signatur = shift || '';
    # das ist ein einziger Scheiss mit dem Crypto-Muell auf CPAN.
    # wir 'signieren' erst mal mit einem hash ueber sich selbst.
    my %verify_sig = ();
    $verify_sig{'verified'} = 0;
    $verify_sig{'errorcode'} = 0;
    $verify_sig{'errormsg'} = '';

    my $sig_new = md5_base64($data);
    if ( $sig_new eq $signatur )
    { $verify_sig{'verified'} = 1; }

    return %verify_sig
}

# my %datenfeld = parse_herstellerID($data);
sub parse_herstellerID
{
    my $data = shift || '';
    my %datenfeld = ();
    my $ende = 0;
    $datenfeld{'errorcode'} = 0;
    $datenfeld{'errormsg'} = '';
    foreach my $line ( split /\n/, $data )
    {
        if ( $line =~ /^\s*$/ ) { next; }
        if ( $line =~ /^\s*\#/ ) { next; }
        if ( $line =~ /^\s*ENDE?\s*$/ ) { $ende = 1; }

	chomp $line;

	if ( $ende ) { $datenfeld{'signature'} .= $line; }
	else
	{
	    my ( $feld, $inhalt ) = split /:\s*/, $line, 2;
	    $datenfeld{$feld} = $inhalt;
	}
    }

    return %datenfeld;
}



###########################################################################
# working subs (generating Files, checking data...)

sub check_antrag
{
    my $data = shift || '';
    my %returnwert = ();

    # init returnwert
    $returnwert{'errorcode'} = 0;
    $returnwert{'errormsg'} = '';

    if ( !$data )
    {
        $returnwert{'errorcode'} = 1;
        $returnwert{'errormsg'} = 'check_antrag: internal error: got no data hashref';
        return %returnwert;
    }


    # each type has its own fields to check for...

    # Umsatzsteuervoranmeldung
    if ( $data->{'type'} eq 'ustva' )
    {
	# has to be present
        foreach my $k ('kz83')
	{
	    if (    ( !defined $data->{'data'}{$k} )
	         || ( $data->{'data'}{$k} =~ /^\s*$/ )
	       )
	    {
                $returnwert{'errorcode'} = 2;
                $returnwert{'errormsg'} .= 'data field: '.$k.' must be defined'."\n";
	    }
	}

	if ( $returnwert{'errorcode'} )
	{ return %returnwert; }

	# has to be a number
        foreach my $k ('kz35', 'kz41', 'kz42', 'kz43', 'kz44',
		       'kz45', 'kz48', 'kz49', 'kz51', 'kz52',
		       'kz57', 'kz60', 'kz73', 'kz76', 'kz77',
		       'kz84', 'kz86', 'kz91', 'kz93', 'kz94',
		       'kz95', 'kz97'
	              )
	{
	    if (    ( defined $data->{'data'}{$k} )
                 && ( $data->{'data'}{$k} )      # nicht 0 und nicht ''
	         && ( $data->{'data'}{$k} !~ /^\d*$/ )
	       )
	    {
                $returnwert{'errorcode'} = 3;
                $returnwert{'errormsg'} .= 'data field: '.$k.' is no number - but has to be one (ganze Zahl)'."\n";
	    }
	}

	if ( $returnwert{'errorcode'} )
	{ return %returnwert; }

	# not negativ
        foreach my $k ('kz39')
	{
	    if (    ( defined $data->{'data'}{$k} )
                 && ( $data->{'data'}{$k} )      # nicht 0 und nicht ''
	         && ( $data->{'data'}{$k} < 0 )
	    )
	    {
                $returnwert{'errorcode'} = 4;
                $returnwert{'errormsg'} .= 'data field: '.$k.' has to be positive or 0 - but not negative'."\n";
	    }
	    
	}

	if ( $returnwert{'errorcode'} )
	{ return %returnwert; }

	# has to be real
        foreach my $k ('kz36', 'kz53', 'kz59', 'kz61', 'kz62',
	               'kz63', 'kz64', 'kz65', 'kz66', 'kz67',
	               'kz69', 'kz74', 'kz80', 'kz83', 'kz85',
	               'kz96', 'kz98'
		       )
	{
	    if (    ( defined $data->{'data'}{$k} )
                 && ( $data->{'data'}{$k} )      # nicht 0 und nicht ''
	         && ( $data->{'data'}{$k} !~ /^\d+\.\d{2}$/ )
	       )
	    {
                $returnwert{'errorcode'} = 5;
                $returnwert{'errormsg'} .= 'data field: '.$k.' is not of the form d.dd - but has to be'."\n";
	    }
	}
    }

    # Umsatzsteuersndervorauszahlung
    elsif ( $data->{'type'} eq 'ustsvz' )
    {
	# kz38 has to be present, a number and not negative
        if ( !defined $data->{'data'}{'kz38'} ) 
	{
                $returnwert{'errorcode'} = 2;
                $returnwert{'errormsg'} .= 'kz38 must be defined'."\n";
	}
	if ( $returnwert{'errorcode'} ) { return %returnwert; }

        if ( $data->{'data'}{'kz38'} !~ /^\d+$/ ) 
	{
                $returnwert{'errorcode'} = 3;
                $returnwert{'errormsg'} .= 'kz38 is no number - but has to be one (ganze Zahl)'."\n";
	}
	if ( $returnwert{'errorcode'} ) { return %returnwert; }

        if ( $data->{'data'}{'kz38'} < 0 ) 
	{
                $returnwert{'errorcode'} = 4;
                $returnwert{'errormsg'} .= 'kz38 has to be positive or 0 - but not negative'."\n";
	}
    }

    # Dauerfriwstverlaengerung
    elsif ( $data->{'type'} eq 'dfv' )
    {
        # only kz09 is allowed!
	# we will regulate this in the TEMPLATE
    }

    # Lohnsteueranmeldung
    elsif ( $data->{'type'} eq 'lsta' )
    {
        # kz83 (0.00), kz86 (0), kz86 nubmer (number of employees) not negat.

        if ( $data->{'data'}{'kz86'} !~ /^\d+$/ ) 
	{
                $returnwert{'errorcode'} = 3;
                $returnwert{'errormsg'} .= 'kz86 is no number - but has to be one (ganze Zahl)'."\n";
	}
	if ( $returnwert{'errorcode'} ) { return %returnwert; }

	foreach my $k ('kz83', 'kz86' )
	{
            if ( $data->{'data'}{$k} < 0 ) 
	    {
                $returnwert{'errorcode'} = 4;
                $returnwert{'errormsg'} .= 'field '.$k.' has to be positive or 0 - but not negative'."\n";
	    }
	}
	if ( $returnwert{'errorcode'} ) { return %returnwert; }

        if ( $data->{'data'}{'kz83'} !~ /^\d+\.\d{2}$/ ) 
	{
                $returnwert{'errorcode'} = 5;
                $returnwert{'errormsg'} .= 'kz83 is not of the form d.dd - but has to be'."\n";
	}
    }
    elsif (  !defined $data->{'type'}  )
    {
        $returnwert{'errorcode'} = 10;
	$returnwert{'errormsg'} .= 'field \'type\'  missing'."\n";
    }
    elsif (  !$data->{'type'}  )
    {
        $returnwert{'errorcode'} = 11;
	$returnwert{'errormsg'} .=  'field \'type\' has no value'."\n";
    }
    else
    {
        $returnwert{'errorcode'} = 12;
	$returnwert{'errormsg'} .=  'unknown field \'type\': \''.$data->{'type'}."'\n";
    }

    return %returnwert;

} # check_antrag

sub normalize_data
{
    my $data = shift || '';
    my %returnwert = ();

    # init returnwert
    $returnwert{'errorcode'} = 0;
    $returnwert{'errormsg'} = '';

    if ( !$data )
    {
        $returnwert{'errorcode'} = 1;
        $returnwert{'errormsg'} = 'normalize_data: internal error: got no data hashref';
        return %returnwert;
    }

    $returnwert{'type'} = $data->{'type'};

    # Umsatzsteuervoranmeldung
    if ( $data->{'type'} eq 'ustva' )
    {
        if ( !defined $data->{'data'}{'kz83'} )
	{ $returnwert{'data'}{'kz83'} = 0.00; }

        foreach my $k ('kz35', 'kz41', 'kz42', 'kz43', 'kz44',
		       'kz45', 'kz48', 'kz49', 'kz51', 'kz52',
		       'kz57', 'kz60', 'kz73', 'kz76', 'kz77',
		       'kz84', 'kz86', 'kz91', 'kz93', 'kz94',
		       'kz95', 'kz97'
	              )
	{
	    if ( defined $data->{'data'}{$k} )
	    { 
	        $data->{'data'}{$k} = 0 if $data->{'data'}{$k} eq '';
	        $returnwert{'data'}{$k} = int($data->{'data'}{$k}); 
            }
	}

        foreach my $k ('kz36', 'kz53', 'kz59', 'kz61', 'kz62',
	               'kz63', 'kz64', 'kz65', 'kz66', 'kz67',
	               'kz69', 'kz74', 'kz80', 'kz83', 'kz85',
	               'kz96', 'kz98'
		       )
	{
	    if ( defined $data->{'data'}{$k} )
	    {
	        # ganze Zahlen: .00 anhaengen 
		# sprintf "%1.2f", $args{'kz83'};
	        $data->{'data'}{$k} = 0 if !$data->{'data'}{$k};
		$returnwert{'data'}{$k} = sprintf "%1.2f", $data->{'data'}{$k};
	    }
	}

    }

    # Umsatzsteuersondervorauszahlung
    if ( $data->{'type'} eq 'ustsvz' )
    {
        if ( !defined $data->{'data'}{'kz38'} )
	{ $returnwert{'data'}{'kz38'} = 0; }

        foreach my $k ('kz35', 'kz38', 'kz41', 'kz42', 'kz43',
	               'kz44', 'kz45', 'kz48', 'kz49', 'kz51',
		       'kz52', 'kz57', 'kz60', 'kz73', 'kz76',
		       'kz77', 'kz84', 'kz86', 'kz91', 'kz93',
		       'kz94', 'kz95', 'kz97'
		       )
	{
	    if ( defined $data->{'data'}{$k} )
	    { 
	        $data->{'data'}{$k} = 0 if !$data->{'data'}{$k};
	        $returnwert{'data'}{$k} = int($data->{'data'}{$k});
	    }
	}

        foreach my $k ('kz36', 'kz53', 'kz59', 'kz61', 'kz62',
	               'kz63', 'kz64', 'kz65', 'kz66', 'kz67',
	               'kz69', 'kz74', 'kz80', 'kz83', 'kz85',
	               'kz96', 'kz98'
		       )
	{
	    if ( defined $data->{'data'}{$k} )
	    {
	        # ganze Zahlen: .00 anhaengen 
		# sprintf "%1.2f", $args{'kz83'};
	        $data->{'data'}{$k} = 0 if !$data->{'data'}{$k};
		$returnwert{'data'}{$k} = sprintf "%1.2f", $data->{'data'}{$k};
	    }
	}
    }

    if ( $data->{'type'} eq 'dfv' )
    {
        # kz09 wird intern zusammengesetzt
    }

    if ( $data->{'type'} eq 'lsta' )
    {
        if ( !defined $data->{'data'}{'kz83'} )
	{ $returnwert{'data'}{'kz83'} = 0.00; }
        if ( !defined $data->{'data'}{'kz86'} )
	{ $returnwert{'data'}{'kz83'} = 0; }
	else 
	{ 
	    $data->{'data'}{'kz86'} = 0 if !$data->{'data'}{'kz86'};
	    $returnwert{'data'}{'kz83'} = int $data->{'data'}{'kz86'};
        }

        foreach my $k ('kz33', 'kz42', 'kz43', 'kz46', 'kz48',
		       'kz49', 'kz61', 'kz62', 'kz63', 'kz64',
		       'kz65', 'kz66', 'kz67', 'kz69', 'kz70',
		       'kz72', 'kz73', 'kz74', 'kz76', 'kz77', 
		       'kz78', 'kz83', 'kz86',
		       )
	{
	    if ( defined $data->{'data'}{$k} )
	    {
	        # ganze Zahlen: .00 anhaengen 
		# sprintf "%1.2f", $args{'kz83'};
	        $data->{'data'}{$k} = 0 if !$data->{'data'}{$k};
		$returnwert{'data'}{$k} = sprintf "%1.2f", $data->{'data'}{$k};
	    }
	}
    }
    return %returnwert;
}


# analog zu externem Programm gen_xml
sub gen_antrag
{
    my $data = shift || '';
    my %returnwert = ();

    # init returnwert
    $returnwert{'errorcode'} = 0;
    $returnwert{'errormsg'} = '';

    if ( !$data )
    {
        $returnwert{'errorcode'} = 1;
        $returnwert{'errormsg'} = 'gen_antrag: internal error: got no data hashref';
        return %returnwert;
    }


    # Backwardcompatibility with first prototyp (type=steuerfall)
    if ( $data->{'type'} )
    {
        $returnwert{'type'} = $data->{'type'};
    }
    elsif ( $data->{'steuerfall'} )
    {
        $returnwert{'type'} = $data->{'steuerfall'};
    }
    else { $returnwert{'type'} = ''; }

    $returnwert{'type'} =~ tr/[A-Z]/[a-z]/;

    # das machen wir mit type.template $template_file setzen
    # default_$type.template
    my $template_file = 'default_XML_'.$returnwert{'type'}.'.template';

    if ( !-f "$TEMPLATES_DIR/$template_file")
    {
        $returnwert{'errorcode'} = 2;
        $returnwert{'errormsg'} = "Templatefile $TEMPLATES_DIR/$template_file nicht gefunden\n";
        return %returnwert;
    }

    # Daten anzeigen
    # define template variables for replacement

# TransferHeader
# datenTeil
# datenFelder
    my $vars = {
                 data => $data,
               };

    # create Template object
    my $template = Template->new($template_config);

    # process input template, substituting variables
    $template->process($template_file, $vars, \$returnwert{'XML'})
                         || die "FEHLER:\n".$template->error();


    return %returnwert;
}

# create blank antrag
sub init_antrag
{
    my $type = shift || '';
    my %returnwert = ();

    # init returnwert
    $returnwert{'errorcode'} = 0;
    $returnwert{'errormsg'} = '';

    if ( !$type )
    {
        $returnwert{'errorcode'} = 1;
        $returnwert{'errormsg'} = 'init_antrag: internal error: got no type';
        return %returnwert;
    }

    my $filename = $TEMPLATES_DIR.'/init_'.$type.'.template';

    if ( !-f $filename)
    {
        $returnwert{'errorcode'} = 3;
        $returnwert{'errormsg'} = 'init_antrag: no such file: '.$filename;
        return %returnwert;
    }


    # Daten initialisieren
    # define template variables for replacement

    open (IN, "$filename") || do {
		    $returnwert{'errorcode'} = 4;
		    $returnwert{'errormsg'} = 'init_antrag: can not open '.$filename.': '.$!."\n"; 
		    return %returnwert;
                };

    while (<IN>) 
    {
        chomp $_;
        if ( /^\s*$/ ) { next; }
        if ( /^\s*\#/ ) { next; }
        my ( $key, $value ) = split /\s*:\s*/, $_, 2;
    
        if ( !defined $key ) { next; }
        elsif ( !$key ) { next; }
        elsif ( !defined $value ) { $value = ''; }
        elsif ( !$value ) { $value = ''; }
        if ( $key =~ /^\s+/ ) { $key =~ s/^\s+//g; }
        if ( $key =~ /\s+$/ ) { $key =~ s/\s+$//g; }
        if ( $value =~ /^\s+/ ) { $value =~ s/^\s+//g; }
        if ( $value =~ /\s+$/ ) { $value =~ s/\s+$//g; }
    
        $returnwert{'data'}{$key} = $value;
    }
    close(IN);

    $returnwert{'type'} = $returnwert{'data'}{'type'};


    return %returnwert;
}

sub send_antrag
{
    my $data = shift || '';
    my %returnwert = ();

    # init returnwert
    $returnwert{'errorcode'} = 0;
    $returnwert{'errormsg'} = '';

    if ( !$data )
    {
        $returnwert{'errorcode'} = 1;
        $returnwert{'errormsg'} = 'send_antrag: internal error: got no data hashref';
        return %returnwert;
    }


# VARIABLEN noch anpassen!!!
    # hier bedienen wir uns vorerst einer externen Java-Methode

    # XML file zum Verschicken ablegen.
    my $ticket = $data->{'data'}{'ticket'};
    open ( OUT, ">$TMP_DIR/elster_$ticket") || do
                       {
		           $returnwert{'errorcode'} = 1;
			   $returnwert{'errormsg'} .= "cannot save to file /tmp/elster_$ticket : $!\n";
		       };
    print OUT $data->{'XML'} if !$returnwert{'errorcode'};
    close OUT;


    if (!$returnwert{'errorcode'} )
    {
        # Configfile schreiben
	open (OUT, ">$COALA_CONFIG_FILE" ) ||
	                print "kann Configfile $COALA_CONFIG_FILE nicht anlegen: $!\n";
	print OUT <<EOF;
[Data]
IMPORT=yes
[Data]
PATH=$TMP_DIR/elster_$ticket
[Info]
Status=01010000
EOF
         close (OUT);
    }
    # ENV Auszug speichern
    open ( OUT , ">>$LOG_DIR\_$ticket\.log" ) || 
                print "kann security-save $LOG_DIR/$ticket\.log nich anlegen: $!\n";
    print OUT
    "\n\n# ------------------------------------------------------\n";

    foreach my $f ( keys %ENV ) { print OUT "$f: $ENV{$f}\n"; }

    print OUT "\n\n".$data->{'XML'};
    print OUT
    "\n# ------------------------------------------------------\n";
    close (OUT);
    
    # Java-Programm starten
    $returnwert{'log'} = '';
    print "CLASSPATH=$ENV{CLASSPATH}\n" if $d;

    print "call: $JAVA $SEND_ANTRAG $TMP_DIR\n" if $d;
    $returnwert{'log'} = `$JAVA $SEND_ANTRAG $TMP_DIR 2>&1`;
    print "Antwort:\n".$returnwert{'log'}."\n" if $d;

    unlink "$COALA_CONFIG_FILE";
    
    # Java-Ergebnis auswerten
    # Nutzdatenheader ->Rueckgabe Code und Text
    if( $returnwert{'log'} =~ 
            /<Nutzdatenblock>.*<Rueckgabe>.*<code>(.*)<\/code>.*<text>(.*)<\/text>.*<\/Rueckgabe>.*<\/Nutzdatenblock>/i )
    {
        $returnwert{'errorcode'} = $1 ;
	$returnwert{'errormsg'} .= $2 || '';
    }

    # Antwort-xml separieren, anpassen und abspeichern (das kann
    # man ausdrucken)
    if ( $returnwert{'log'} =~ /Antwort:\s*(.*)$/ )
    {
        $returnwert{'ELSTER_XML'} = $1;
	# Modifikationen:
	$returnwert{'ELSTER_XML'} =~ s/<HerstellerID>(\d+)<\/HerstellerID>//g;
	$returnwert{'ELSTER_XML'} =~ s/\s+xmlns=\"[^"]+\"//g;
        # spaeter in der jeweiligen GUI machen
	# $returnwert{'ELSTER_XML'} =~ s/<Elster/<?xml-stylesheet type="text\/xsl" href="\/websteuer\/ustva.xsl"?><Elster/g;
    }


    return %returnwert;
}


sub read_antrag
{
    my $data = shift || '';
    my %returnwert = ();
    my $filename = '';

    # init returnwert
    $returnwert{'errorcode'} = 0;
    $returnwert{'errormsg'} = '';

    if ( !$data )
    {
        $returnwert{'errorcode'} = 1;
        $returnwert{'errormsg'} = 'read_antrag: internal error: got no data hashref';
        return %returnwert;
    }

    if ( !$data->{'type'} && !$data->{'file'} && !$data->{'import'} )
    {
        $returnwert{'errorcode'} = 2;
        $returnwert{'errormsg'} = 'read_antrag: internal error: got no type';
        return %returnwert;
    }

    if ( $data->{'import'} )
    { $filename = $data->{'file'}; }
    else
    {
	# ACHTUNG: fuer Windows-User: hier mal ein $seperator einfhuehren!
	# XXXHIER
        $filename = $ARCHIV_DIR.'/'.$data->{'user'}.'/'.$data->{'type'}.'_'.$data->{'file'};
    }

    # XXXHIER bei Verwendung von DB, das hier anders machen!
    if ( !-f $filename )
    {
        $returnwert{'errorcode'} = 3;
        $returnwert{'errormsg'} = 'read_antrag: no such file: '.$filename;

        return %returnwert;
    }

    open (IN, "$filename") || do {
		    $returnwert{'errorcode'} = 4;
		    $returnwert{'errormsg'} = 'read_antrag: can not open '.$filename.': '.$!."\n"; 
		    return %returnwert;
                };

    while (<IN>) 
    {
        chomp $_;
        if ( /^\s*$/ ) { next; }
        if ( /^\s*\#/ ) { next; }
        my ( $key, $value ) = split /\s*:\s*/, $_, 2;
    
        if ( !defined $key ) { next; }
        elsif ( !$key ) { next; }
        elsif ( !defined $value ) { $value = ''; }
        elsif ( !$value ) { $value = ''; }
        if ( $key =~ /^\s+/ ) { $key =~ s/^\s+//g; }
        if ( $key =~ /\s+$/ ) { $key =~ s/\s+$//g; }
        if ( $value =~ /^\s+/ ) { $value =~ s/^\s+//g; }
        if ( $value =~ /\s+$/ ) { $value =~ s/\s+$//g; }
    
        $returnwert{'data'}{$key} = $value;
    }
    close(IN);

    # Backwards compatibility
    if ( !$returnwert{'data'}{'type'} && $returnwert{'data'}{'steuerfall'} )
    {
        $returnwert{'data'}{'type'} = $returnwert{'data'}{'steuerfall'};
    }
    $returnwert{'type'} = $returnwert{'data'}{'type'};

    return %returnwert;
}


#XXXXHIER
sub save_antrag
{
    my $data = shift || '';
    my %returnwert = ();
    my $filename = '';
    my $antrag = '';

    # init returnwert
    $returnwert{'errorcode'} = 0;
    $returnwert{'errormsg'} = '';

    if ( !$data )
    {
        $returnwert{'errorcode'} = 1;
        $returnwert{'errormsg'} = 'save_antrag: internal error: got no data hashref';
        return %returnwert;
    }

    if ( !$data->{'type'} )
    {
        $returnwert{'errorcode'} = 2;
        $returnwert{'errormsg'} = 'save_antrag: internal error: got no type';
        return %returnwert;
    }


    # create antrag for saving via template

    # das machen wir mit type.template $template_file setzen
    # default_$type.template
    my $template_file = 'default_ARCHIV_'.$data->{'type'}.'.template';

    if ( !-f "$TEMPLATES_DIR/$template_file")
    {
        $returnwert{'errorcode'} = 2;
        $returnwert{'errormsg'} = "Templatefile $TEMPLATES_DIR/$template_file nicht gefunden\n";
        return %returnwert;
    }

# XXXX noch nicht komplett sauber durchprogrammiert! ERRORhandling! TEST!

    # Daten anzeigen
    # define template variables for replacement
    my $vars = {
                 data => $data->{'data'}
               };

    # create Template object
    my $template = Template->new($template_config);

    # process input template, substituting variables
    $template->process($template_file, $vars, \$antrag)
                         || die "FEHLER:\n".$template->error();



    if ( $data->{'export'} )
    { $filename = $data->{'file'}; }
    else
    {
	# ACHTUNG: fuer Windows-User: hier mal ein $seperator einfhuehren!
	# XXXHIER
        $filename = $ARCHIV_DIR.'/'.$data->{'user'}.'/'.$data->{'type'}.'_'.$data->{'file'};
    }

    open (OUT, ">$filename") || do {
		    $returnwert{'errorcode'} = 4;
		    $returnwert{'errormsg'} = 'save_antrag: can not save '.$filename.': '.$!."\n"; 
		    return %returnwert;
                };

    print OUT $antrag;

    close(OUT);


    return %returnwert;
}


sub read_user_config
{
    my $data = shift || '';
    my %returnwert = ();
    my $filename = '';

    # init returnwert
    $returnwert{'errorcode'} = 0;
    $returnwert{'errormsg'} = '';

    if ( !$data )
    {
        $returnwert{'errorcode'} = 1;
        $returnwert{'errormsg'} = 'read_user_config: internal error: got no data hashref';
        return %returnwert;
    }

    if ( !$data->{'user'} )
    {
        $returnwert{'errorcode'} = 2;
        $returnwert{'errormsg'} = 'read_user_config: internal error: got no user';
        return %returnwert;
    }

    if ( $data->{'file'} )
    { $filename = $USER_CONFIG_DIR.'/'.$data->{'user'}.'/'.$data->{'file'}; }


    if ( $data->{'file'} )
    {
        if ( !-f $filename )
	{
            $returnwert{'errorcode'} = 3;
            $returnwert{'errormsg'} = 'read_user_config: File $filename not found';
	    return %returnwert;
	}

        open (IN, "$filename" ) || do
               { 
                   $returnwert{'errorcode'} = 4;
                   $returnwert{'errorcode'} = "read_user_config: could not open Configfile $filename : $!\n";
                   return %returnwert;
               };


        while(<IN>)
        {
            chomp $_;
            if ( /^\s*$/ ) { next; }
            if ( /^\s*\#/ ) { next; }
            my ( $key, $value ) = split /\s*:\s*/, $_, 2;
    
            if ( !defined $key ) { next; }
            elsif ( !$key ) { next; }
            elsif ( !defined $value ) { $value = ''; }
            elsif ( !$value ) { $value = ''; }
            if ( $key =~ /^\s+/ ) { $key =~ s/^\s+//g; }
            if ( $key =~ /\s+$/ ) { $key =~ s/\s+$//g; }
            if ( $value =~ /^\s+/ ) { $value =~ s/^\s+//g; }
            if ( $value =~ /\s+$/ ) { $value =~ s/\s+$//g; }
    
            $returnwert{'data'}{$key} = $value;
        }
        close(IN);
    }
    else
    {
        $returnwert{'errorcode'} = 10;
        $returnwert{'errormsg'} = 'read_user_config: get user config from db not implemented';
    }

    return %returnwert;
}


sub get_user_permission
{
    my $data = shift || '';
    my %returnwert = ();
    my $filename = '';

    # init returnwert
    $returnwert{'errorcode'} = 0;
    $returnwert{'errormsg'} = '';

    if ( !$data )
    {
        $returnwert{'errorcode'} = 1;
        $returnwert{'errormsg'} = 'get_user_permission: internal error: got no data hashref';
        return %returnwert;
    }

    if ( !$data->{'user'} )
    {
        $returnwert{'errorcode'} = 2;
        $returnwert{'errormsg'} = 'get_user_permission: internal error: got no user';
        return %returnwert;
    }

    if ( !$data->{'action'} )
    {
        $returnwert{'errorcode'} = 3;
        $returnwert{'errormsg'} = 'get_user_permission: internal error: got no action';
        return %returnwert;
    }

    # db nutzen
    my $dbh = open_db_sqlite($DB_FILE);
    if ( $dbh =~ /^ERROR:/ ) 
    {
        $returnwert{'errorcode'} = 4;
        $returnwert{'errormsg'} = 'get_user_permission: internal error: ERROR'.$dbh."\n";
        return %returnwert;
    }

    # get permission from database (table permission)
    # for user/action-combination


    return %returnwert;
}

# ######################################################################
1;
