#!/usr/bin/perl -w

# CVS: $Id: db_sqlite.pm,v 1.1.1.1 2005/08/30 16:42:10 neko Exp $

package db_sqlite;

use Exporter;
use Fcntl;
use DBI;

# debugging on/off
my $d = 0;

@ISA = qw(Exporter);


@EXPORT = qw(
	     close_db
	     delete_entry
	     get_felder
	     new_entry
	     open_db_sqlite
	     open_db_sqlite2
	     update_entry
	     );

########################################################################

# close_db
# Syntax: my $error = close_db($dbh)
sub close_db
{
    my $dbh = shift;
    my $returnwert = 0;

    $dbh->disconnect();

    return $returnwert;
}

# delete_entry
# Syntax: my $error = delete_db($dbh, $tables, $where)
sub delete_entry
{
    my $dbh = shift;
    my $tables = shift;
    my $where = shift;
    my $returnwert = 0;

    if ( $where =~ /^\d+$/ )
    { $where = "id=\'$where\'"; }

print STDERR "call: DELETE FROM $tables WHERE $where\n" if $d;
        $query =<<EOF;
DELETE FROM $tables WHERE $where
EOF

# print "call: $query\n";
    $sth = $dbh->prepare($query);

    $sth->execute() or return "ERROR: Cannot execute: " . $sth->errstr();
    # close_db();

    $sth->finish();
    return $returnwert;
}

# get_felder
# Syntax: my @ergebnis = get_felder($dbh, $tables, $felder, $where, $order_by)
# @ergebnis ist ein Array of hashrefs
# nach dem ersten Feld wird sortiert
sub get_felder
{
    my $dbh = shift || '';
    my $tables = shift || '';
    my $felder = shift || '';
    my $where = shift || '';
    my $order_by = shift || '';
    my @returnwert = ();
# $d = 1;
print STDERR "<br>order_by: $order_by\n" if $d;
    my @tmp_felder = split /,/, $felder;

    if (    (defined $tmp_felder[0]) 
         && ($tmp_felder[0] ne '')
	 && ($tmp_felder[0] ne '*')
         && ($order_by eq '')
       )
    { $order_by = "$tmp_felder[0]"; }

    if ( $where =~ /^\d+$/ ) { $where = "id=\"$where\""; }

    $order_by = "ORDER BY $order_by" if ($order_by ne '');

    my $WHERE = '';
    if ( $where !~ /^\s*$/ ) { $WHERE = 'WHERE '.$where; }
   
print STDERR "<br>-->>order_by: $order_by\n" if $d;
    my $query = "SELECT $felder FROM $tables $WHERE $order_by";
    print STDERR "<br>get_felder call: $query\n" if $d;
    my $sth = $dbh->prepare($query);

    $sth->execute() or return "ERROR: Cannot execute: " . $sth->errstr();

    # Daten aufsammeln und in ein Array verfrachten
    while (my $row = $sth->fetchrow_hashref) 
    {
        my %tmp = ();

# $d = 1; print STDERR "<pre>";
	foreach my $k ( keys %{$row} )
	{
	    print STDERR "Feld: $k: ".$row->{$k}."\n" if $d;
	    if ( defined $row->{$k} ) { $tmp{$k} = $row->{$k}; }
            else { $tmp{$k} = ''; }
	}
# print "row->id: $row->{'id'}\n";

# $d = 0; print "<\pre>";
        push @returnwert, \%tmp;
    }

    return @returnwert;
}

# new_entry
# Syntax: my $id = new_entry($dbh, $table, $hashref)
# liefert die ID oder einen String, der mit ERROR beginnt
# 1. ID organisieren
# 2. Feld mit der ID belegen

sub new_entry
{
    my $dbh = shift;
    my $table = shift;
    my $daten_ref = shift;
    $new_id = 0;

    if ( !$dbh ) { return "ERROR: new_entry hat keine dbh bekommen\n"; }
    if ( !$table ) { return "ERROR: new_entry hat keine table bekommen\n"; }
    if ( !$daten_ref ) { return "ERROR: new_entry hat keine daten bekommen\n"; }


    my $query = "SELECT max(id) FROM $table";
print "new_entry call: $query\n" if $d;
    $sth = $dbh->prepare($query);
    $sth->execute() or return "ERROR: Cannot execute: " . $sth->errstr();

    while (my $row = $sth->fetchrow_hashref)
    { 

# foreach $k (keys %{$row} ) { print $k.": ".$row->{$k}."\n"; }
    
# print "Kontrollpkt1 ".$row->{'max(id)'}."\n" if $d;
        if ( defined ($row->{'max(id)'})
	     and ( $row->{'max(id)'} !~ /^\s*$/ )
	   )
        { 
print STDERR "gefunden: ".$row->{'max(id)'}."\n" if $d;
            $new_id = $row->{'max(id)'}; last; 
        }
    }
    $new_id++;

    $sth->finish();

    $query =<<EOF;
INSERT INTO $table (id) VALUES ($new_id)
EOF
   print "new_entry call: $query\n" if $d;

# print "stelle query zusammen: $query\n";
# print "call: $query\n";
    # $sth = $dbh->prepare("INSERT INTO $table (id VALUES ('$new_id'"); 

    $sth = $dbh->prepare($query);

    $sth->execute() or return "ERROR: Cannot execute: " . $sth->errstr();
    # close_db();

    # jetzt noch das update
    $daten_ref->{'id'} = $new_id;
    my $erg = update_entry($dbh, $table, $daten_ref, $new_id);
    if ( $erg =~ /^ERROR/i ) { $new_id = $erg."new_id: $new_id"; }
print STDERR "$erg\n" if $d;

# print "get_new_id liefere ID $new_id aus\n";


    return $new_id;
}




# this may change to SQLite - whatever this will be today...
# open_db_sqlite
# Syntax: my $dbh = open_db_sqlite_db($db_name)
sub open_db_sqlite
{
    my $db_name = shift;

    my $dbh = DBI->connect("dbi:SQLite:dbname=$db_name","","")
            or return "ERROR: Cannot connect: " . $DBI::errstr;

    return $dbh;
}

# open_db_sqlite2
# Syntax: my $dbh = open_db_sqlite2($db_name)
sub open_db_sqlite2
{
    my $db_name = shift;

    my $dbh = DBI->connect("dbi:SQLite2:dbname=$db_name","","")
            or return "ERROR: Cannot connect: " . $DBI::errstr;

    return $dbh;
}



# update_entry
# Syntax: my $error = delete_db($dbh, $tables, $hashref, $where)
# wenn $where lediglich eine Zahl ist, wird sie als ID des 
# Datensatzes gewertet.

sub update_entry
{
    my $dbh = shift;
    my $tables = shift;
    my $daten_ref = shift;
    my $where = shift;
    my $returnwert = 0;
    my $set = '';

# $d=1;
    # seems to be a bug in sqlite3 - the column is not updated if 
    # number is surrounded by ' or "
    if ( $where =~ /^\d+$/ ) { $where = "id=$where"; }

    # $set aus Datenref zusammenstellen
$returnwert .= "Daten speichern: Tabelle: $tables, where: $where<pre>\n" if $d;
    foreach my $k ( keys %{$daten_ref} )
    {
	if ( $set ne '' ) { $set .= ', '; }
	$set .= "$k='$daten_ref->{$k}'";
$returnwert .= "$k: $daten_ref->{$k}\n" if $d;
    }
$returnwert .= "</pre>\n" if $d;

    my $query = "UPDATE $tables SET $set WHERE $where";
    $returnwert .= "update_entry call: $query\n" if $d;

    my $sth = $dbh->prepare($query);
    $sth->execute() or $returnwert .= "ERROR: Cannot execute: " . $sth->errstr();
    $sth->finish();

    return $returnwert;

}




########################################################################
1;

# $Log: db_sqlite.pm,v $
# Revision 1.1.1.1  2005/08/30 16:42:10  neko
# Modularisierte Variante von Websteuer
#
# Saubere Trennung zwischen GUI/(Re)Praesentation und arbeitenden
# Funktionen.
#
# Ziel: verschiedene GUI-Varianten und Speichern oder nicht etc...
#

